"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const proto_1 = require("./proto");
const meta_1 = require("./meta");
const helpers_1 = require("./helpers");
exports.getMetadata = (model, realPath) => {
    // field metadata flow
    const proto = proto_1.getProto(model.form, realPath);
    const propName = realPath[realPath.length - 1];
    if (!proto) {
        return undefined;
    }
    const meta = meta_1.getMetadataField(proto, propName);
    if (meta) {
        return meta;
    }
    const protoClass = proto_1.getProto(model.form, realPath.concat(undefined));
    if (!protoClass) {
        return undefined;
    }
    // class metadata flow
    const protoForm = proto_1.getProtoForm(protoClass);
    return meta_1.getMetadataForm(protoForm);
};
function createReconsiler(options, realPath) {
    const handler = {
        get(_, method) {
            const newRealPath = realPath.concat(method);
            return createReconsiler(options, newRealPath);
        },
    };
    if (!realPath) {
        return new Proxy({}, handler);
    }
    const renderer = (model, opts) => {
        var _a;
        const fieldMeta = exports.getMetadata(model, realPath);
        const meta = Object.assign(Object.assign({}, fieldMeta), opts === null || opts === void 0 ? void 0 : opts.meta);
        const Component = (_a = opts === null || opts === void 0 ? void 0 : opts.Component) !== null && _a !== void 0 ? _a : options.resolveComponent(meta === null || meta === void 0 ? void 0 : meta.type);
        if (!Component) {
            return null;
        }
        const name = helpers_1.getInputName(model.form, realPath, model.parentFormName);
        return react_1.default.createElement(Component, {
            model,
            name,
            path: realPath,
            options,
            meta: meta
        });
    };
    return new Proxy(renderer, handler);
}
function reconsiler(props) {
    return createReconsiler(props, []);
}
exports.reconsiler = reconsiler;
//# sourceMappingURL=form.js.map